#Niche Modeling Workshop 
#Data Cleaning Script

#install the packages we need (only do it once)
install.packages("data.table")
install.packages("dplyr")
install.packages("sp")
install.packages("maptools")
install.packages("spatstat")
install.packages("rgeos")
install.packages("rgdal")
install.packages("raster")

#load all the libraries we will need here (each time you oepn the script)
library(data.table)
library(dplyr)
library(sp)
library(maptools)
library(spatstat)
library(rgeos)
library(rgdal)
library(raster)

#set the working directory
#setwd("~/Desktop/_______")

#reformat the downloaded raw data to just include species name, latitude, longitude, 
#and year collected, similar to the SampleFL-data.csv

#load the dataset 
raw.data <- read.csv(file="datasets/SampleFL-data.csv", header=TRUE)
head(raw.data)
raw.data$species

#show unique species names
unique(raw.data$species)
length(unique(raw.data$species))

##############Taxonomic Name Resolution#############
#first make a list of all taxa in our dataset
mynames <- unique(raw.data$species)
write.csv(mynames, "mynames.csv")

#We will now use the online iPlant TNRS Tool. Upload your taxa names to the TNRS website 
#and download the results  
#These results are the tnrs_results.txt file

iplants <- read.table("tnrs_results.txt", sep= '\t', header=TRUE)
head(iplants)
corr.names <- cbind(as.character(iplants$Name_matched), as.character(iplants$Accepted_name))
head(corr.names)
corr.names <- corr.names[-1,]
colnames(corr.names) <- c("species", "new")
head(corr.names)
head(raw.data)
new.names <- gsub(" ", "\\_", corr.names)
head(new.names)

#combine the corrected names with the raw data
work.data <- raw.data
head(work.data)
work.data$species <- as.character(work.data$species)
m <- merge(work.data, new.names, by = "species")
head(m)
m$species <- m$new
m$new <- NULL
head(m)
unique(m$species)
length(unique(m$species))

#Let's look at the date range for our collections
unique(m$year)
m$year <- gsub("\\N", NA, m$year)
unique(m$year)
m$year <- as.numeric(m$year)
unique(m$year)
min.age <- min(na.omit(m$year))
min.age
max.age <- max(na.omit(m$year))
max.age

# if you want to set a minimum year threshold, set it here

n <- subset(m, year >= 1975)

##################Rounding up latitude/longitude precision############
# We need to think about how fine or coarse our maps will be.
# Look at the distance each decimal place represents (at the equator)
# decimal
# places   degrees          distance
# -------  -------          --------
# 0        1                111  km
# 1        0.1              11.1 km
# 2        0.01             1.11 km
# 3        0.001            111  m
# 4        0.0001           11.1 m
# How much area do you want each record to encompass?  Change accordingly

n$lat <- round(n$lat, digits = 2)
n$lat
n$long <- round(n$long, digits = 2)

###############Cleaning special cases of long or lat of 0
dim(n)
n1 <- subset(n, n$lat != 0.00)
n1 <- subset(n1, n1$long != 0.00)
dim(n1)

#################Removing duplicate rows
n.unique <- n1 %>% distinct
dim(n.unique)

###############Clipping to a shapefile
#Upload a shapefile (the World in the example) to view the country borders
World <- readShapePoly("TM_WORLD_BORDERS_SIMPL-0.3/TM_WORLD_BORDERS_SIMPL-0.3.shp", proj4string=CRS("+proj=longlat +datum=WGS84 +no_defs"))
plot(World)

#Look at the points and see how some are out of place
points <- data.frame(x = n.unique$long, y = n.unique$lat)
points <- SpatialPoints(points)
plot(points, add=TRUE, pch=19, col="purple")

#make sure the plot viewer is open and completely viewable, 
#use drawExtent to select the two corners of your desired extent
extent <- drawExtent()
plot(zoom(World, extent), add=TRUE)
cropped <- crop(World, extent)
writeSpatialShape(cropped, "Cropped")
plot(points, add=TRUE, pch=19, col="purple")

###rather than designating your own extent, you can get country shapefiles as seen here
# USAMap <- getData("GADM", country="USA", level=0)
# cropped <- plot(USAMap)
# writeSpatialShape(cropped, "Cropped")
# plot(points, add=TRUE, pch=19, col="purple")

#extract the values of the points for the extent
proj4string(points)=CRS("+proj=longlat +datum=WGS84 +no_defs")
over2 <- over(points, cropped)

#recombine everything to gather all data
total <- cbind(n.unique, over2)
total

#Get rid of points that fell outside extent (will have "NA" instead of a polygon)
PointsWithinRegions <-total[!is.na(total$LON), ]
head(PointsWithinRegions)

#We need a csv with only species, lat, long for MaxEnt input
FinalDataset <- cbind(as.character(PointsWithinRegions$species), PointsWithinRegions$long, PointsWithinRegions$lat)
row.names(FinalDataset)<-NULL
dim(FinalDataset)
head(FinalDataset)
write.csv(FinalDataset, "MaxEntPointsInput.csv", row.names = FALSE)
